@@ -28,21 +28,25 @@ class RSSResultSummarizer:
         self,
         elements,
         result_paths,
-        use_joblib,
+        use_joblib: bool = True,
         num_process: int = -1,
         backend: str = "loky",
+        symprec_set: list[float] = [1e-5, 1e-4, 1e-3, 1e-2],
         output_poscar: bool = False,
-        threshold: float = None,
+        thresholds: list[float] = None,
         parse_vasp: bool = False,
+        update_result: bool = False,
     ):
         self.elements = elements
         self.result_paths = result_paths
         self.use_joblib = use_joblib
         self.num_process = num_process
         self.backend = backend
+        self.symprec_set = symprec_set
         self.output_poscar = output_poscar
-        self.threshold = threshold
+        self.thresholds = thresholds
         self.parse_vasp = parse_vasp
+        self.update_result = update_result
 
         self.num_opt_struct = 0
         self.pressure = None
@@ -54,8 +58,12 @@ class RSSResultSummarizer:
 
         if not self.parse_vasp:
             paths_same_comp, results_same_comp = self._parse_mlp_result()
+            axis_tol = 0.01
+            pos_tol = 0.01
         else:
             paths_same_comp, results_same_comp = self._parse_vasp_result()
+            axis_tol = 0.05
+            pos_tol = 0.03
 
         for comp_ratio, res_paths in paths_same_comp.items():
             self.num_opt_struct = 0
@@ -76,7 +84,12 @@ class RSSResultSummarizer:
             res_paths, integrated_res_paths = self.initialize_uniq_struct(
                 yaml_name, res_paths
             )
-            self.sort_in_single_comp(res_paths, results_same_comp[comp_ratio])
+            self.sort_in_single_comp(
+                res_paths,
+                results_same_comp[comp_ratio],
+                axis_tol=axis_tol,
+                pos_tol=pos_tol,
+            )
 
             time_finish = time() - time_start
 
@@ -110,7 +123,7 @@ class RSSResultSummarizer:
                         print(log_name, file=f)
                         print(np.round(ghost_minima_info[1], 3), file=f)
             else:
-                is_ghost_minima = np.full(energies.shape, False, dtype=bool)
+                is_ghost_minima = None
 
             rss_result_all = log_unique_structures(
                 log_name + ".yaml",
@@ -122,8 +135,12 @@ class RSSResultSummarizer:
             with open(f"json/{log_name}.json", "w") as f:
                 json.dump(rss_result_all, f)
 
-            if self.output_poscar:
-                self.generate_poscars(f"json/{log_name}.json", threshold=self.threshold)
+            if self.thresholds is not None or self.output_poscar is not False:
+                self.generate_poscars(
+                    f"json/{log_name}.json",
+                    thresholds=self.thresholds,
+                    output_poscar=self.output_poscar,
+                )
 
             print(log_name, "finished", flush=True)
 
@@ -143,7 +160,11 @@ class RSSResultSummarizer:
 
             time_start = time()
             self.sort_in_single_comp(
-                res_paths, results_same_comp[comp_ratio], keep_unique=True
+                res_paths,
+                results_same_comp[comp_ratio],
+                keep_unique=True,
+                axis_tol=0.1,
+                pos_tol=0.03,
             )
             time_finish = time() - time_start
 
@@ -173,23 +194,161 @@ class RSSResultSummarizer:
             with open(f"json/{log_name}.json", "w") as f:
                 json.dump(rss_result_all, f)
 
-            if self.output_poscar:
-                self.generate_poscars(f"json/{log_name}.json", threshold=self.threshold)
+            if self.thresholds is not None or self.output_poscar is not False:
+                self.generate_poscars(
+                    f"json/{log_name}.json",
+                    thresholds=self.thresholds,
+                    output_poscar=self.output_poscar,
+                )
 
             print(log_name, "finished", flush=True)
 
+    def sort_in_single_comp(
+        self,
+        result_paths,
+        rss_result_dict,
+        keep_unique=False,
+        axis_tol=0.01,
+        pos_tol=0.01,
+    ):
+        rss_results = []
+        for res_path in result_paths:
+            loaded_dict = rss_result_dict[res_path]
+            rss_res = loaded_dict["rss_results"]
+            if not self.parse_vasp:
+                for res in rss_res:
+                    res["structure"] = polymlp_struct_from_dict(res["structure"])
+                    res["struct_no"] = None
+
+            pressure = loaded_dict.get("pressure")
+            for res in rss_res:
+                res["pressure"] = pressure
+            self.pressure = pressure
+
+            rss_results.extend(rss_res)
+
+        unique_structs = generate_unique_structs(
+            rss_results,
+            use_joblib=self.use_joblib,
+            num_process=self.num_process,
+            backend=self.backend,
+            symprec_set1=self.symprec_set,
+        )
+        self.num_opt_struct += len(unique_structs)
+
+        for unique_struct in unique_structs:
+            self.analyzer.identify_duplicate_struct(
+                unique_struct=unique_struct,
+                keep_unique=keep_unique,
+                axis_tol=axis_tol,
+                pos_tol=pos_tol,
+            )
+
+    def initialize_uniq_struct(self, yaml_name, result_paths):
+        pre_result_paths = []
+
+        if os.path.isfile(yaml_name):
+            with open(yaml_name) as f:
+                yaml_data = yaml.safe_load(f)
+            self.num_opt_struct = yaml_data["general_information"][
+                "num_optimized_structs"
+            ]
+            pre_result_paths = yaml_data["general_information"]["input_file_names"]
+            if self.update_result:
+                pre_result_paths = []
+
+            with open(f"./json/{yaml_name.split('.yaml')[0]}.json") as f:
+                loaded_dict = json.load(f)
+            rss_results = loaded_dict["rss_results"]
+            for i in range(len(rss_results)):
+                rss_results[i]["structure"] = polymlp_struct_from_dict(
+                    rss_results[i]["structure"]
+                )
+            self.pressure = loaded_dict["pressure"]
+
+            unique_structs = generate_unique_structs(
+                rss_results,
+                use_joblib=self.use_joblib,
+                num_process=self.num_process,
+                backend=self.backend,
+                symprec_set1=self.symprec_set,
+            )
+            self.analyzer._initialize_unique_structs(unique_structs)
+
+        not_processed_path = list(set(result_paths) - set(pre_result_paths))
+        integrated_res_paths = list(set(result_paths) | set(pre_result_paths))
+
+        return not_processed_path, integrated_res_paths
+
+    def generate_poscars(self, json_path: str, thresholds=None, output_poscar=False):
+        if thresholds is None:
+            thresholds = [None]
+
+        struct_counts = []
+        logname = os.path.basename(json_path).split(".json")[0]
+        for threshold in thresholds:
+            if threshold is None:
+                dir_name = "poscars"
+            else:
+                threshold = float(threshold)
+                dir_name = f"poscars_{threshold}"
+            if output_poscar:
+                os.makedirs(f"{dir_name}/{logname}", exist_ok=True)
+
+            print(f"Threshold (meV/atom): {threshold}")
+
+            with open(json_path) as f:
+                loaded_dict = json.load(f)
+            rss_results = loaded_dict["rss_results"]
+
+            e_min = None
+            struct_count = 0
+            for res in rss_results:
+                if not res.get("is_ghost_minima") and e_min is None:
+                    e_min = res["energy"]
+                elif res.get("is_ghost_minima") and output_poscar:
+                    dest = f"{dir_name}/{logname}/POSCAR_{logname}_No{res['struct_no']}"
+                    shutil.copy(res["poscar"], dest)
+                    continue
+
+                if e_min is not None and threshold is not None:
+                    diff = abs(e_min - res["energy"])
+                    if diff * 1000 > threshold:
+                        continue
+
+                if output_poscar:
+                    dest = f"{dir_name}/{logname}/POSCAR_{logname}_No{res['struct_no']}"
+                    shutil.copy(res["poscar"], dest)
+                struct_count += 1
+
+            struct_counts.append(struct_count)
+            print("Number of local minimum structures:", struct_count)
+
+        return struct_counts
+
     def _parse_mlp_result(self):
         paths_same_comp = defaultdict(list)
         results_same_comp = defaultdict(dict)
         for path_name in self.result_paths:
-            rss_result_path = f"{path_name}/rss_result/rss_results.json"
+            rss_result_path = f"{path_name}"
             with open(rss_result_path) as f:
                 loaded_dict = json.load(f)
 
-            rel_path = os.path.relpath(f"{path_name}/opt_struct", start=os.getcwd())
             for i in range(len(loaded_dict["rss_results"])):
-                poscar_name = loaded_dict["rss_results"][i]["poscar"].split("/")[-1]
-                loaded_dict["rss_results"][i]["poscar"] = f"{rel_path}/{poscar_name}"
+                if "opt_struct" not in loaded_dict["rss_results"][i]["poscar"]:
+                    _path_name = "/".join(path_name.split("/")[:-2])
+                    rel_path = os.path.relpath(
+                        f"{_path_name}/opt_struct", start=os.getcwd()
+                    )
+                    poscar_name = loaded_dict["rss_results"][i]["poscar"].split("/")[-1]
+                    poscar_path = f"{rel_path}/{poscar_name}"
+                else:
+                    _path_name = "/".join(path_name.split("/")[:-2])
+                    poscar_path = os.path.relpath(
+                        f'{_path_name}/{loaded_dict["rss_results"][i]["poscar"]}',
+                        start=os.getcwd(),
+                    )
+                loaded_dict["rss_results"][i]["poscar"] = poscar_path
 
             target_elements = loaded_dict["elements"]
             comp_ratio = tuple(loaded_dict["comp_ratio"])
@@ -225,6 +384,10 @@ class RSSResultSummarizer:
                 energy_dft -= atomic_energy(element)
             energy_dft /= len(polymlp_st.elements)
 
+            if energy_dft < -5:
+                print(path_name, "exhibits an unphysically low energy. Skipping.")
+                continue
+
             res_dict["poscar"] = path_name + "/vasprun.xml"
             res_dict["structure"] = polymlp_st
             res_dict["energy"] = energy_dft
@@ -267,86 +430,3 @@ class RSSResultSummarizer:
             results_same_comp[comp_ratio_orderd][path_name] = loaded_dict
 
         return paths_same_comp, results_same_comp
-
-    def sort_in_single_comp(self, result_paths, rss_result_dict, keep_unique=False):
-        rss_results = []
-        for res_path in result_paths:
-            loaded_dict = rss_result_dict[res_path]
-            rss_res = loaded_dict["rss_results"]
-            if not self.parse_vasp:
-                for res in rss_res:
-                    res["structure"] = polymlp_struct_from_dict(res["structure"])
-
-            pressure = loaded_dict.get("pressure")
-            for res in rss_res:
-                res["pressure"] = pressure
-            self.pressure = pressure
-
-            rss_results.extend(rss_res)
-
-        unique_structs = generate_unique_structs(
-            rss_results,
-            use_joblib=self.use_joblib,
-            num_process=self.num_process,
-            backend=self.backend,
-        )
-        self.num_opt_struct += len(unique_structs)
-
-        for unique_struct in unique_structs:
-            self.analyzer.identify_duplicate_struct(
-                unique_struct=unique_struct,
-                keep_unique=keep_unique,
-            )
-
-    def initialize_uniq_struct(self, yaml_name, result_paths):
-        pre_result_paths = []
-
-        if os.path.isfile(yaml_name):
-            with open(yaml_name) as f:
-                yaml_data = yaml.safe_load(f)
-            self.num_opt_struct = yaml_data["general_information"][
-                "num_optimized_structs"
-            ]
-            pre_result_paths = yaml_data["general_information"]["input_file_names"]
-
-            with open(f"./json/{yaml_name.split('.yaml')[0]}.json") as f:
-                loaded_dict = json.load(f)
-            rss_results = loaded_dict["rss_results"]
-            for i in range(len(rss_results)):
-                rss_results[i]["structure"] = polymlp_struct_from_dict(
-                    rss_results[i]["structure"]
-                )
-            self.pressure = loaded_dict["pressure"]
-
-            unique_structs = generate_unique_structs(
-                rss_results,
-                use_joblib=self.use_joblib,
-                num_process=self.num_process,
-                backend=self.backend,
-            )
-            self.analyzer._initialize_unique_structs(unique_structs)
-
-        not_processed_path = list(set(result_paths) - set(pre_result_paths))
-        integrated_res_paths = list(set(result_paths) | set(pre_result_paths))
-
-        return not_processed_path, integrated_res_paths
-
-    def generate_poscars(self, json_path: str, threshold=None):
-        logname = os.path.basename(json_path).split(".json")[0]
-        os.makedirs(f"poscars/{logname}", exist_ok=True)
-
-        with open(json_path) as f:
-            loaded_dict = json.load(f)
-        rss_results = loaded_dict["rss_results"]
-
-        e_min = None
-        for res in rss_results:
-            if not res.get("is_ghost_minima") and e_min is None:
-                e_min = res["energy"]
-            if e_min is not None and threshold is not None:
-                diff = abs(e_min - res["energy"])
-                if diff * 1000 > threshold:
-                    continue
-
-            dest = f"poscars/{logname}/POSCAR_{logname}_No{res['struct_no']}"
-            shutil.copy(res["poscar"], dest)
