@@ -8,12 +8,11 @@ import numpy as np
 from pypolymlp.core.data_format import PolymlpStructure
 from pypolymlp.core.interface_vasp import Poscar
 from rsspolymlp.analysis.struct_matcher.struct_match import (
-    IrrepStructure,
-    generate_irrep_struct,
+    ReducedStructRep,
     generate_primitive_cells,
+    generate_reduced_struct,
     struct_match,
 )
-from rsspolymlp.analysis.struct_matcher.utils import get_recommend_symprecs
 from rsspolymlp.common.composition import compute_composition
 from rsspolymlp.common.convert_dict import polymlp_struct_to_dict
 from rsspolymlp.common.property import PropUtil
@@ -21,8 +20,8 @@ from rsspolymlp.common.property import PropUtil
 
 @dataclass
 class UniqueStructure:
-    irrep_struct_set: list[IrrepStructure]
-    recommend_symprecs: list[float]
+    reduced_struct_set: list[ReducedStructRep]
+    symprec_set: list[float]
     original_structure: PolymlpStructure
     axis_abc: np.ndarray
     n_atoms: int
@@ -90,39 +89,48 @@ class UniqueStructureAnalyzer:
         is_change_struct = False
         _energy = unique_struct.energy
         _spg_list = unique_struct.spg_list
-        _irrep_struct_set = unique_struct.irrep_struct_set
+        _reduced_struct_set = unique_struct.reduced_struct_set
         if other_properties is None:
             other_properties = {}
 
-        for idx, ndstr in enumerate(self.unique_str):
-            if use_energy_spg_check:
-                if abs(ndstr.energy - _energy) < energy_diff and any(
-                    spg in _spg_list for spg in ndstr.spg_list
+        for idx, _ndstr in enumerate(self.unique_str):
+            targets = self.unique_str_keep[idx] if keep_unique else [_ndstr]
+
+            for ndstr in targets:
+                if use_energy_spg_check:
+                    same_energy = abs(ndstr.energy - _energy) < energy_diff
+                    shared_spg = any(spg in _spg_list for spg in ndstr.spg_list)
+
+                    if same_energy and shared_spg:
+                        is_unique = False
+                        if self._spg_count(_spg_list) > self._spg_count(ndstr.spg_list):
+                            is_change_struct = True
+                        break
+
+                if struct_match(
+                    ndstr.reduced_struct_set,
+                    _reduced_struct_set,
+                    axis_tol=axis_tol,
+                    pos_tol=pos_tol,
                 ):
                     is_unique = False
-                    if self._extract_spg_count(_spg_list) > self._extract_spg_count(
-                        ndstr.spg_list
+                    if self._spg_count(_spg_list) > self._spg_count(ndstr.spg_list) or (
+                        self._spg_count(_spg_list) == self._spg_count(ndstr.spg_list)
+                        and _energy is not None
+                        and ndstr.energy is not None
+                        and _energy < ndstr.energy
                     ):
                         is_change_struct = True
                     break
 
-            if struct_match(
-                ndstr.irrep_struct_set,
-                _irrep_struct_set,
-                axis_tol=axis_tol,
-                pos_tol=pos_tol,
-            ):
-                is_unique = False
-                if self._extract_spg_count(_spg_list) > self._extract_spg_count(
-                    ndstr.spg_list
-                ):
-                    is_change_struct = True
+            if not is_unique:
                 break
 
         if not is_unique:
             if is_change_struct:
                 # Update duplicate count and replace with better data if necessary
                 unique_struct.dup_count = self.unique_str[idx].dup_count
+                unique_struct.struct_no = self.unique_str[idx].struct_no
                 self.unique_str[idx] = unique_struct
                 self.unique_str_prop[idx] = other_properties
             self.unique_str[idx].dup_count += 1
@@ -136,9 +144,12 @@ class UniqueStructureAnalyzer:
                 self.unique_str_keep.append([unique_struct])
                 self.unique_str_prop_keep.append([other_properties])
 
+        if is_unique and len(self.unique_str) % 500 == 0:
+            print(f"Reached {len(self.unique_str)} unique structures.")
+
         return is_unique, is_change_struct
 
-    def _extract_spg_count(self, spg_list):
+    def _spg_count(self, spg_list):
         """Extract and sum space group counts from a list of space group strings."""
         return sum(
             int(re.search(r"\((\d+)\)", s).group(1))
@@ -167,7 +178,11 @@ def generate_unique_struct(
     spg_list: Optional[list[str]] = None,
     pressure: Optional[float] = None,
     struct_no: Optional[int] = None,
-    symprec_set: list[float] = [1e-5, 1e-4, 1e-3, 1e-2],
+    dup_count: int = 1,
+    symprec_set1: list[float] = [1e-5, 1e-4, 1e-3, 1e-2],
+    symprec_set2: list[float] = [1e-4, 1e-2, 1e-1],
+    standardize_axis: bool = False,
+    cartesian_coords: bool = True,
 ) -> UniqueStructure:
     """
     Generate a UniqueStructure object.
@@ -216,26 +231,34 @@ def generate_unique_struct(
 
     primitive_st_set, spg_number_set = generate_primitive_cells(
         polymlp_st=polymlp_st,
-        symprec_set=symprec_set,
+        symprec_set=symprec_set1,
     )
     if primitive_st_set == []:
         return None
 
-    irrep_struct_set = []
+    reduced_struct_set = []
     for i, primitive_st in enumerate(primitive_st_set):
-        recommend_symprecs = get_recommend_symprecs(primitive_st)
-        irrep_struct = generate_irrep_struct(
+        symprec_set2 = sorted(
+            symprec_set2,
+            key=lambda x: x if isinstance(x, (int, float)) else sum(x) / len(x),
+        )
+
+        reduced_struct = generate_reduced_struct(
             primitive_st,
             spg_number_set[i],
-            symprec_irreps=[1e-5] + recommend_symprecs,
+            symprec_set=symprec_set2,
+            standardize_axis=standardize_axis,
+            cartesian_coords=cartesian_coords,
         )
-        irrep_struct_set.append(irrep_struct)
+        reduced_struct_set.append(reduced_struct)
 
     objprop = PropUtil(polymlp_st.axis.T, polymlp_st.positions.T)
+    if spg_list is None:
+        spg_list = objprop.analyze_space_group(polymlp_st.elements)
 
     return UniqueStructure(
-        irrep_struct_set=irrep_struct_set,
-        recommend_symprecs=recommend_symprecs,
+        reduced_struct_set=reduced_struct_set,
+        symprec_set=[symprec_set1, symprec_set2],
         original_structure=polymlp_st,
         axis_abc=objprop.abc,
         n_atoms=int(len(polymlp_st.elements)),
@@ -246,11 +269,19 @@ def generate_unique_struct(
         pressure=pressure,
         input_poscar=poscar_name,
         struct_no=struct_no,
+        dup_count=dup_count,
     )
 
 
 def generate_unique_structs(
-    rss_results, use_joblib=True, num_process=-1, backend="loky"
+    rss_results,
+    use_joblib: bool = True,
+    num_process: int = -1,
+    backend: str = "loky",
+    symprec_set1: list[float] = [1e-5, 1e-4, 1e-3, 1e-2],
+    symprec_set2: list[float] = [1e-4, 1e-2, 1e-1],
+    standardize_axis: bool = False,
+    cartesian_coords: bool = True,
 ) -> list[UniqueStructure]:
     """
     Generate a list of UniqueStructure objects from the given RSS results.
@@ -259,8 +290,9 @@ def generate_unique_structs(
     ----------
     rss_results : list of dict
         A list of dictionaries, where each dictionary contains a single structure information.
-        Each dictionary must include the following keys:
-            - "poscar": structure in POSCAR format (str or Poscar object)
+        Each dictionary must include "poscar" keys:
+            - "poscar": path of POSCAR format file
+        Optional keys:
             - "structure": PolymlpStructure object
             - "energy": total energy (float)
             - "spg_list": list of space group symbols identified under multiple tolerances
@@ -272,6 +304,8 @@ def generate_unique_structs(
         The number of parallel jobs. -1 means using all available processors.
     backend : str, default="loky"
         Backend used by joblib.
+    symprec_set : list of float, default=[1e-5, 1e-4, 1e-3, 1e-2]
+        Symmetry tolerances used to determine distinct primitive cells.
 
     Returns
     -------
@@ -282,11 +316,16 @@ def generate_unique_structs(
         unique_structs = joblib.Parallel(n_jobs=num_process, backend=backend)(
             joblib.delayed(generate_unique_struct)(
                 poscar_name=res["poscar"],
-                polymlp_st=res["structure"],
-                energy=res["energy"],
-                spg_list=res["spg_list"],
+                polymlp_st=res.get("structure", None),
+                energy=res.get("energy", None),
+                spg_list=res.get("spg_list", None),
                 pressure=res.get("pressure", None),
                 struct_no=res.get("struct_no", None),
+                dup_count=res.get("dup_count", 1),
+                symprec_set1=symprec_set1,
+                symprec_set2=symprec_set2,
+                standardize_axis=standardize_axis,
+                cartesian_coords=cartesian_coords,
             )
             for res in rss_results
         )
@@ -296,11 +335,16 @@ def generate_unique_structs(
             unique_structs.append(
                 generate_unique_struct(
                     poscar_name=res["poscar"],
-                    polymlp_st=res["structure"],
-                    energy=res["energy"],
-                    spg_list=res["spg_list"],
+                    polymlp_st=res.get("structure", None),
+                    energy=res.get("energy", None),
+                    spg_list=res.get("spg_list", None),
                     pressure=res.get("pressure", None),
                     struct_no=res.get("struct_no", None),
+                    dup_count=res.get("dup_count", 1),
+                    symprec_set1=symprec_set1,
+                    symprec_set2=symprec_set2,
+                    standardize_axis=standardize_axis,
+                    cartesian_coords=cartesian_coords,
                 )
             )
     unique_structs = [s for s in unique_structs if s is not None]
@@ -308,12 +352,19 @@ def generate_unique_structs(
 
 
 def log_unique_structures(
-    file_name,
-    unique_structs,
-    is_ghost_minima,
+    file_name: str,
+    unique_structs: list[UniqueStructure],
+    is_ghost_minima=None,
     pressure=None,
     unique_struct_iters=None,
 ):
+    if is_ghost_minima is None:
+        is_ghost_minima = np.full_like(unique_structs, False, dtype=bool)
+    for i in range(len(unique_structs)):
+        if not is_ghost_minima[i]:
+            energy_min = unique_structs[i].energy
+            break
+
     struct_num_max = max(
         (_s.struct_no for _s in unique_structs if _s.struct_no is not None), default=0
     )
@@ -322,60 +373,68 @@ def log_unique_structures(
             struct_num_max += 1
             _s.struct_no = struct_num_max
 
-    for i in range(len(unique_structs)):
-        if not is_ghost_minima[i]:
-            energy_min = unique_structs[i].energy
-            break
-
     rss_results = []
     with open(file_name, "a") as f:
         print("unique_structures:", file=f)
-        for idx, _str in enumerate(unique_structs):
-            e_diff = round((_str.energy - energy_min) * 1000, 2)
-            print(f"  - struct_No: {_str.struct_no}", file=f)
-            print(f"    poscar_name: {_str.input_poscar}", file=f)
-            print(f"    energy_diff_meV_per_atom: {e_diff}", file=f)
-            print(f"    duplicates: {_str.dup_count}", file=f)
-            print(f"    enthalpy: {_str.energy}", file=f)
-            print(f"    axis: {_str.axis_abc}", file=f)
-            print(
-                f"    positions: {_str.original_structure.positions.T.tolist()}", file=f
-            )
-            print(f"    elements: {_str.original_structure.elements}", file=f)
-            print(f"    space_group: {_str.spg_list}", file=f)
 
-            info = [
-                f"{_str.n_atoms} atom",
-                f"distance {round(_str.least_distance, 3)} (Ang.)",
-                f"volume {round(_str.volume, 2)} (A^3/atom)",
-            ]
-            if unique_struct_iters is not None:
-                info.append(f"iteration {unique_struct_iters[idx]}")
-            print(f"    other_info: {' / '.join(info)}", file=f)
+        for is_ghost in [False, True]:
+            for idx, _str in enumerate(unique_structs):
+                if energy_min is not None:
+                    e_diff = round((_str.energy - energy_min) * 1000, 2)
+                    if (not is_ghost and e_diff < -300) or (
+                        is_ghost and e_diff >= -300
+                    ):
+                        continue
+                elif is_ghost:
+                    continue
+                print(f"  - struct_No: {_str.struct_no}", file=f)
+                print(f"    poscar_name: {_str.input_poscar}", file=f)
+                if energy_min is not None:
+                    print(f"    energy_diff_meV_per_atom: {e_diff}", file=f)
+                print(f"    duplicates: {_str.dup_count}", file=f)
+                print(f"    enthalpy: {_str.energy}", file=f)
+                print(f"    axis: {_str.axis_abc}", file=f)
+                print(
+                    f"    positions: {_str.original_structure.positions.T.tolist()}",
+                    file=f,
+                )
+                print(f"    elements: {_str.original_structure.elements}", file=f)
+                print(f"    space_group: {_str.spg_list}", file=f)
 
-            if is_ghost_minima[idx]:
-                print("    ghost_minima_flag: true", file=f)
+                info = [
+                    f"{_str.n_atoms} atom",
+                    f"distance {round(_str.least_distance, 3)} (Ang.)",
+                    f"volume {round(_str.volume, 2)} (A^3/atom)",
+                ]
+                if unique_struct_iters is not None:
+                    info.append(f"iteration {unique_struct_iters[idx]}")
+                print(f"    other_info: {' / '.join(info)}", file=f)
 
-            _res = {}
-            _res["poscar"] = _str.input_poscar
-            polymlp_st = _str.original_structure
-            polymlp_st_dict = polymlp_struct_to_dict(polymlp_st)
-            _res["structure"] = polymlp_st_dict
-            _res["energy"] = _str.energy
-            _res["pressure"] = pressure
-            _res["spg_list"] = _str.spg_list
-            _res["struct_no"] = _str.struct_no
-            _res["is_ghost_minima"] = bool(is_ghost_minima[idx])
-            rss_results.append(_res)
+                if is_ghost_minima[idx]:
+                    print("    ghost_minima_flag: true", file=f)
 
-    comp_res = compute_composition(unique_structs[0].original_structure.elements)
-
-    rss_result_all = {
-        "elements": comp_res.unique_elements.tolist(),
-        "comp_ratio": comp_res.comp_ratio,
-        "pressure": pressure,
-        "rss_results": rss_results,
-    }
+                rss_results.append(
+                    {
+                        "poscar": _str.input_poscar,
+                        "structure": polymlp_struct_to_dict(_str.original_structure),
+                        "energy": _str.energy,
+                        "pressure": pressure,
+                        "spg_list": _str.spg_list,
+                        "struct_no": _str.struct_no,
+                        "dup_count": _str.dup_count,
+                        "is_ghost_minima": bool(is_ghost_minima[idx]),
+                    }
+                )
+    if len(unique_structs) > 0:
+        comp_res = compute_composition(unique_structs[0].original_structure.elements)
+        rss_result_all = {
+            "elements": comp_res.unique_elements.tolist(),
+            "comp_ratio": comp_res.comp_ratio,
+            "pressure": pressure,
+            "rss_results": rss_results,
+        }
+    else:
+        rss_result_all = {}
 
     return rss_result_all
 
@@ -383,6 +442,7 @@ def log_unique_structures(
 def log_all_unique_structures(
     file_name,
     unique_structs,
+    unique_structs_prop=None,
 ):
     rss_results = []
     with open(file_name, "a") as f:
@@ -421,6 +481,8 @@ def log_all_unique_structures(
                 _res["struct_no"] = f"{idx1 + 1}_{idx2+1}"
                 _res["is_ghost_minima"] = False
                 rss_results.append(_res)
+            if unique_structs_prop is not None:
+                print("    properties:", unique_structs_prop[idx1], file=f)
 
     comp_res = compute_composition(unique_structs[0][0].original_structure.elements)
 
