@@ -15,7 +15,6 @@ import numpy as np
 from pypolymlp.core.interface_vasp import Poscar
 from pypolymlp.core.io_polymlp import load_mlps
 from rsspolymlp.analysis.ghost_minima import detect_ghost_minima
-from rsspolymlp.analysis.struct_matcher.utils import get_distance_cluster
 from rsspolymlp.analysis.unique_struct import (
     UniqueStructureAnalyzer,
     generate_unique_structs,
@@ -35,6 +34,9 @@ class RSSResultAnalyzer:
         self.iter_str = []  # Iteration statistics
         self.fval_str = []  # Function evaluation statistics
         self.gval_str = []  # Gradient evaluation statistics
+        self.total_iter_str = 0
+        self.total_fval_str = 0
+        self.total_gval_str = 0
         self.errors = Counter()  # Error tracking
         self.error_poscar = defaultdict(list)  # POSCAR error details
         self.time_all = 0  # Total computation time accumulator
@@ -81,6 +83,10 @@ class RSSResultAnalyzer:
     def _read_and_validate_logfile(self, logfile):
         try:
             struct_prop, judge = LogfileLoader(logfile).read_file()
+            self.time_all += struct_prop["time"]
+            self.total_iter_str += struct_prop["iter"]
+            self.total_fval_str += struct_prop["fval"]
+            self.total_gval_str += struct_prop["gval"]
         except (TypeError, ValueError):
             self.errors["else_err"] += 1
             self.error_poscar["else_err"].append(
@@ -109,7 +115,6 @@ class RSSResultAnalyzer:
             self.error_poscar["else_err"].append(poscar_name)
             return None, None
 
-        self.time_all += struct_prop["time"]
         self.potential = struct_prop["potential"]
         if struct_prop["pressure"] is not None:
             self.pressure = struct_prop["pressure"]
@@ -128,7 +133,7 @@ class RSSResultAnalyzer:
                     cutoff_i = model_dict["model"]["cutoff"]
                     if cutoff_i > max_cutoff:
                         max_cutoff = cutoff_i
-                self.cutoff = max_cutoff
+                self.cutoff = max_cutoff - 0.5
 
         polymlp_st = Poscar(poscar_name).structure
         objprop = PropUtil(polymlp_st.axis.T, polymlp_st.positions.T)
@@ -136,17 +141,19 @@ class RSSResultAnalyzer:
         _struct_prop = struct_prop
         _struct_prop["structure"] = polymlp_st
 
-        distance_cluster = get_distance_cluster(polymlp_st=polymlp_st)
-        if distance_cluster is not None:
-            max_layer_diff = max(
-                [
-                    np.max(distance_cluster[0]) * axis_abc[0],
-                    np.max(distance_cluster[1]) * axis_abc[1],
-                    np.max(distance_cluster[2]) * axis_abc[2],
-                ]
-            )
-            if max_layer_diff > self.cutoff:
-                return None
+        positions = polymlp_st.positions.T
+        if positions.shape[0] == 1:
+            max_gap = axis_abc[0:3]
+        else:
+            sort_idx = np.argsort(positions, axis=0, kind="mergesort")
+            coord_sorted = np.take_along_axis(positions, sort_idx, axis=0)
+            gap = np.roll(coord_sorted, -1, axis=0) - coord_sorted
+            gap[-1, :] += 1.0
+            gap = gap * axis_abc[0:3]
+            max_gap = np.max(gap, axis=0)
+        max_layer_diff = np.max(max_gap)
+        if max_layer_diff > self.cutoff:
+            return None
 
         return _struct_prop
 
@@ -215,11 +222,15 @@ class RSSResultAnalyzer:
             finished_set = finished_set[: index + 1]
         self.logfiles = [f"log/{p}.log" for p in finished_set]
 
+        print("Loading RSS results...")
         struct_properties = self._load_rss_logfiles()
+        print("RSS results loaded")
 
+        print("Eliminating duplicate structures...")
         unique_structs, unique_str_prop = self._analysis_unique_structure(
             struct_properties, use_joblib, num_process, backend
         )
+        print("Duplicate structures eliminated")
 
         time_finish = time() - time_start
 
@@ -257,15 +268,16 @@ class RSSResultAnalyzer:
             print(f"  max_rss_structures:       {max_init_str}", file=f)
             print(f"  num_initial_structures:   {finish_count}", file=f)
             print(f"  num_optimized_structures: {success_count}", file=f)
+            print(f"  num_unique_structures:    {len(unique_structs)}", file=f)
             print(f"  stopping_criterion:       {stop_mes}", file=f)
             print(f"  optimized_per_initial:    {prop_success}", file=f)
             print(f"  total_rss_time_sec:       {int(self.time_all)}", file=f)
             print("", file=f)
 
             print("evaluation_counts:", file=f)
-            print(f"  iteration:            {self.iter_str[-1]}", file=f)
-            print(f"  function_evaluations: {self.fval_str[-1]}", file=f)
-            print(f"  gradient_evaluations: {self.gval_str[-1]}", file=f)
+            print(f"  iteration:            {self.total_iter_str}", file=f)
+            print(f"  function_evaluations: {self.total_fval_str}", file=f)
+            print(f"  gradient_evaluations: {self.total_gval_str}", file=f)
             print("", file=f)
 
             print("error_counts:", file=f)
@@ -301,21 +313,23 @@ class RSSResultAnalyzer:
                 energies[sort_idx], distances[sort_idx]
             )
         else:
-            is_ghost_minima = np.full_like(energies, False, dtype=bool)
+            is_ghost_minima = None
 
         rss_result_all = log_unique_structures(
             file_name, unique_str_sorted, is_ghost_minima, self.pressure, iters_sorted
         )
-        with open("rss_result/rss_results.json", "w") as f:
-            json.dump(rss_result_all, f)
+        if not rss_result_all == {}:
+            with open("rss_result/rss_results.json", "w") as f:
+                json.dump(rss_result_all, f)
 
         with open(file_name, "a") as f:
             print("", file=f)
-            print("evaluation_count_per_structure:", file=f)
-            print(f"  iteration_list:            {self.iter_str}", file=f)
-            print(f"  function_evaluations_list: {self.fval_str}", file=f)
-            print(f"  gradient_evaluations_list: {self.gval_str}", file=f)
-            print("", file=f)
+            if len(self.iter_str) > 0:
+                print("evaluation_count_per_structure:", file=f)
+                print(f"  iteration_list:            {self.iter_str}", file=f)
+                print(f"  function_evaluations_list: {self.fval_str}", file=f)
+                print(f"  gradient_evaluations_list: {self.gval_str}", file=f)
+                print("", file=f)
 
             print("poscar_names_failed:", file=f)
             print(f"  low_energy:       {self.error_poscar['energy_low']}", file=f)
